C===================================================================
C
      PROGRAM AEMELBE
C
C  This program solves the plane stress or plane strain problem 
C  of an inhomogeneous anisotropic body using the Analog Equation Method 
C  using constant boundary elements 
C
   	USE MSIMSL
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)                     
      CHARACTER*15 INPUTFILE,OUTPUTFILE
C
C  Set the maximum dimensions
C
      PARAMETER (N=200) 
      PARAMETER (IN=81) 
	PARAMETER (NB=1)
C               
C  N=  Number of boundary elements equal to number of boundary 
C      nodes 
C  IN= Number of internal collocation points 
C
C  NB= Number of boundaries of the multiple boundary domain
C
	COMMON C
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N),XIN(IN),YIN(IN),ICN(N+1)
	1          ,NL(NB) 
      DIMENSION C1(N,N),C2(N,N),C3(N),D1(N,N),D2(N,N),D3(N)
	DIMENSION FX(IN),FY(IN)
	DIMENSION C11B(N),C12B(N),C13B(N),C22B(N),C23B(N),C33B(N)
	DIMENSION C11(IN),C12(IN),C13(IN),C22(IN),C23(IN),C33(IN)
	DIMENSION C11X(IN),C12X(IN),C13X(IN),C22X(IN),C23X(IN),C33X(IN)
	DIMENSION C11Y(IN),C12Y(IN),C13Y(IN),C22Y(IN),C23Y(IN),C33Y(IN)
	DIMENSION D(N,N),ENX(N),ENY(N)

	DIMENSION G(N,N),H(N,N) 
      DIMENSION UHAT(N,IN),QHAT(N,IN),T(N,IN)
      DIMENSION GIN(IN,N),GX(IN,N),GY(IN,N), 
     1          GXX(IN,N),GXY(IN,N),GYY(IN,N)
	DIMENSION HIN(IN,N),HX(IN,N),HY(IN,N),
     1          HXX(IN,N),HXY(IN,N),HYY(IN,N)
	DIMENSION UINHAT(IN,IN),UXHAT(IN,IN),UYHAT(IN,IN),
     1          UXXHAT(IN,IN),UXYHAT(IN,IN),UYYHAT(IN,IN)
 	DIMENSION Z(4*N,2*IN),E(4*N)

	DIMENSION SU1(IN,IN),SUX1(IN,IN),SUY1(IN,IN),SUXX1(IN,IN)
	1         ,SUXY1(IN,IN),SUYY1(IN,IN)
	DIMENSION SU2(IN,IN),SUX2(IN,IN),SUY2(IN,IN),SUXX2(IN,IN)
	1         ,SUXY2(IN,IN),SUYY2(IN,IN)
	DIMENSION ZU(IN),ZUX(IN),ZUY(IN),ZUXX(IN),ZUXY(IN),ZUYY(IN)

	DIMENSION SV1(IN,IN),SVX1(IN,IN),SVY1(IN,IN),SVXX1(IN,IN)
 	1         ,SVXY1(IN,IN),SVYY1(IN,IN)
	DIMENSION SV2(IN,IN),SVX2(IN,IN),SVY2(IN,IN),SVXX2(IN,IN)
	1         ,SVXY2(IN,IN),SVYY2(IN,IN)
	DIMENSION ZV(IN),ZVX(IN),ZVY(IN),ZVXX(IN),ZVXY(IN),ZVYY(IN)

      DIMENSION U(IN),UB(N),V(IN),VB(N)
	DIMENSION SX(IN),SY(IN),SXY(IN),SXB(N),SYB(N),SXYB(N)
	1         ,STX(N),STY(N)
C
C  XL,YL     = Coordinates of the extreme points of the boundary elements
C  XM,YM     = Coordinates of the midpoints of the boundary elements
C  XIN,YIN   = Coordinates of the internal collocation points 
C  UB,QB     = Boundary nodal values of u and un
C  G,H       = Influence matrices Gij and Hij relating the boundary points
C  UHAT,QHAT = Influence matrices resulting from the particular solution Uhat
C           	 and its normal derivative Qhat relating the internal 
C              points and the boundary nodal points
C
C  GIN,GX,GY,GINXX,GXY,GYY 
C		   = Influence matrx Gij and its derivatives relating 
C              the internal points and the boundary nodal points
C
C  HIN,HX,HY,HX,HXY,HYY 
C            = Influence matrix Hij and its derivatives relating 
C              the internal points and the boundary nodal points
C
C  UINHAT,UXHAT,UYHAT,UXXHAT,UXYHAT,UYYHAT
C            = Influence matrix Uij and its derivatives relating 
C              the internal points 
C

C	   
C  Read the names and open the input and output files
C   
	WRITE (*,'(A)')' Name of the INPUTFILE (max.15 characters)'
      READ  (*,'(A)') INPUTFILE
      WRITE (*,'(A)')' Name of the OUTPUTFILE (max.15 characters)'
      READ  (*,'(A)') OUTPUTFILE
      OPEN (UNIT=1, FILE=inputfile)
      OPEN (UNIT=2, FILE=OUTputfile)	  
	READ (*,*) C
C
C  Read data from INPUTFILE
C
	CALL INPUT (XL,YL,XIN,YIN,NL,C1,C2,C3,D1,D2,D3,FX,FY,C11,C12,
 	1   C13,C22,C23,C33,C11B,C12B,C13B,C22B,C23B,C33B,C11X,C12X,C13X,
     1   C22X,C23X,C33X,C11Y,C12Y,C13Y,C22Y,C23Y,C33Y,N,IN,NB,ICN,D,
     1   ENX,ENY) 

      WRITE(*,*)('FFFFF 1 FFFFF')
C
C  Compute the G matrix
C
      CALL GMATR (XL,YL,XM,YM,G,N,NL,NB) 
C
      WRITE(*,*)('FFFFF 2 FFFFF')
C         
C  Compute the H matrix
C
      CALL HMATR (XL,YL,XM,YM,H,N,NL,NB)
C
      WRITE(*,*)('FFFFF 3 FFFFF')
C
C  Compute the  matrices UHAT,QHAT and T
C
      CALL UQTMATR (XL,YL,XM,YM,XIN,YIN,G,H,UHAT,QHAT,T,N,IN,NL,NB
	1     ,ENX,ENY)

      WRITE(*,*)('FFFFF 4 FFFFF')
C
C  Compute the matrix Z and the vector e 
C
      CALL BOUNDMATR(G,H,T,C1,C2,C3,D1,D2,D3,C11B,C12B,C13B,C22B,C23B
	1      ,C33B,ENX,ENY,D,N,IN,Z,E) 

      WRITE(*,*)('FFFFF 5 FFFFF')
C
C  Compute the matrices GIN, Gx,Gy,...,Hin,..Hyy
C
      CALL GHINMATR(XL,YL,XIN,YIN,GIN,GX,GY,GXX,GXY,GYY,
     1	          HIN,HX,HY,HXX,HXY,HYY,ENX,ENY,N,IN,NL,NB)

      WRITE(*,*)('FFFFF 6 FFFFF')
C
C  Compute the  matrices UHATIN,UHATx,UHATy,....,UHATyy
C
      CALL UINHATMATR(XIN,YIN,UINHAT,UXHAT,UYHAT,
     1	                      UXXHAT,UXYHAT,UYYHAT,IN)

      WRITE(*,*)('FFFFF 7 FFFFF')
C
C  Compute the  matrices Su1,Su1x,Su1y,Su1xx,Su1xy,Su1yy
C                        Su2,Su2x,Su2y,Su2xx,Su2xy,Su2yy
C                        Sv1,Sv1x,Sv1y,Sv1xx,Sv1xy,Sv1yy
C                        Sv2,Sv2x,Sv2y,Sv2xx,Sv2xy,Sv2yy
C  and the vectors       zu,zux,zuy,zuxx,zuxy,zuyy
C                        zv,zvx,zvy,zvxx,zvxy,zvyy

      CALL SZMATR(GIN,GX,GY,GXX,GXY,GYY,HIN,HX,HY,HXX,HXY,HYY,UINHAT
 	1 ,UXHAT,UYHAT,UXXHAT,UXYHAT,UYYHAT,UHAT,QHAT,Z,E,N,IN
     1 ,SU1,SUX1,SUY1,SUXX1,SUXY1,SUYY1,SU2,SUX2,SUY2,SUXX2,SUXY2,SUYY2
     1 ,SV1,SVX1,SVY1,SVXX1,SVXY1,SVYY1,SV2,SVX2,SVY2,SVXX2,SVXY2,SVYY2
     1 ,ZU,ZUX,ZUY,ZUXX,ZUXY,ZUYY,ZV,ZVX,ZVY,ZVXX,ZVXY,ZVYY)
 	     
      WRITE(*,*)('FFFFF 8 FFFFF')
C
C  Compute the values of the solution U,V,Sx,Sy,Sxy at domain and 
C  boundary points 
C 

	CALL SOLVE(Z,E,SU1,SUX1,SUY1,SUXX1,SUXY1,SUYY1,SU2,SUX2
 	1 ,SUY2,SUXX2,SUXY2,SUYY2,SV1,SVX1,SVY1,SVXX1,SVXY1,SVYY1,SV2
     1 ,SVX2,SVY2,SVXX2,SVXY2,SVYY2,ZU,ZUX,ZUY,ZUXX,ZUXY,ZUYY,ZV,ZVX
     1 ,ZVY,ZVXX,ZVXY,ZVYY,FX,FY,C11,C12,C13,C22,C23,C33,C11B,C12B
     1 ,C13B,C22B,C23B,C33B,C11X,C12X,C13X,C22X,C23X,C33X,C11Y,C12Y
     1 ,C13Y,C22Y,C23Y,C33Y,N,IN,ENX,ENY,D,U,UB,V,VB,SX,SY,SXY
     1 ,SXB,SYB,SXYB,STX,STY)

      WRITE(*,*)('FFFFF  9  FFFFF')
C
C  Print the results in the OUTPUTFILE
C
      CALL OUTPUT(XM,YM,XIN,YIN,U,UB,V,VB,SX,SY,SXY
     1 ,SXB,SYB,SXYB,STX,STY,N,IN)
C
      WRITE(*,*)('FFFFFF 10 FFFFF')
C
C  Close input and output files
C
      CLOSE(1)
      CLOSE(2)
      STOP
      END
C      
C====================================================================
C
      SUBROUTINE INPUT (XL,YL,XIN,YIN,NL,C1,C2,C3,D1,D2,D3,FX,FY,C11,
	1  C12,C13,C22,C23,C33,C11B,C12B,C13B,C22B,C23B,C33B,C11X,C12X,
     1  C13X,C22X,C23X,C33X,C11Y,C12Y,C13Y,C22Y,C23Y,C33Y,N,IN,NB,ICN,
     1  D,ENX,ENY)   
C     
C  This subroutine reads the input data from the input file
C  and writes them in the output file 
C
      IMPLICIT  DOUBLEPRECISION (A-H,O-Z)
      CHARACTER*80 NAME,TITLE
      DIMENSION XL(N+1),YL(N+1),XIN(IN),YIN(IN),ICN(n+1),NL(NB)
      DIMENSION C1(N,N),C2(N,N),C3(N),D1(N,N),D2(N,N),D3(N)
	DIMENSION FX(IN),FY(IN)
	DIMENSION C11B(N),C12B(N),C13B(N),C22B(N),C23B(N),C33B(N)
	DIMENSION C11(IN),C12(IN),C13(IN),C22(IN),C23(IN),C33(IN)
	DIMENSION C11X(IN),C12X(IN),C13X(IN),C22X(IN),C23X(IN),C33X(IN)
	DIMENSION C11Y(IN),C12Y(IN),C13Y(IN),C22Y(IN),C23Y(IN),C33Y(IN)
	DIMENSION D(N,N),ENX(N),ENY(N),DX(N),DY(N)
	DIMENSION DT(N,N),XL1(N+1),YL1(N+1),ICN1(N+1)

      WRITE(2,100)
  100 FORMAT(' ',79('*'))
C
C  Read user's name
C
      READ(1,'(A)')NAME
      WRITE(2,'(A)')NAME
C
C  Read the title of the program
C
      READ(1,'(A)')TITLE
      WRITE(2,'(A)')TITLE
      WRITE(2,200)N,IN,NB
  200 FORMAT(//'BASIC PARAMETERS'//2X,'NUMBER OF BOUNDARY ELEMENTS='
     1,I4/2X,'NUMBER OF INTERNAL COLLOCATION POINTS =',I5/2X,
     1'NUMBER OF BOUNDARIES   =',I3)
C
C  Read the last element at each boundary
C
	READ (1,*) (NL(I),I=1,NB)
C
C  Read the coordinates of the extreme points of the boundary elements XL,YL
C
      READ(1,*) (XL(I),YL(I),icn(i),I=1,N)


C
C  Computation of the direction cosines ofthe normal unit vector n
C

	IF (NB.GT.1) GOTO 2
	XL(N+1)=XL(1)
	YL(N+1)=YL(1)

	ENX=0.
	ENY=0.

    2	DO 5 I=1,N
	DX(I)=XL(I+1)-XL(I)    
    5 DY(I)=YL(I+1)-YL(I)

	IF (NB.LE.1) GOTO 15
	DX(NL(1))=XL(1)-XL(NL(1))
	DY(NL(1))=YL(1)-YL(NL(1))
	DO 10 K=2,NB
	DX(NL(K))=XL(NL(K-1)+1)-XL(NL(K))
   10 DY(NL(K))=YL(NL(K-1)+1)-YL(NL(K))

   15	DO 20 I=1,N
      SL=DSQRT(DX(I)**2+DY(I)**2)
      ENX(I)=DY(I)/SL
   20 ENY(I)=-DX(I)/SL
C   20 WRITE (2,*) ENX(I),ENY(I)
C	STOP
C
C  Write the coordinates in the output file
C
      WRITE(2,300)
  300 FORMAT(//2X,'COORDINATES OF THE EXTREME POINTS OF THE BOUNDARY ELE
     1MENTS',//2X,'POINT',9X,'XL',15X,'YL')
      DO 25 I=1,N
   25 WRITE(2,400) I,XL(I),YL(I)
  400 FORMAT(2X,I3,2(3X,E14.5))

C
C  Read the coefficients of the boundary conditions c1,c2,c3,d1,d2,d3 of Eq. (8.131) 
C
C
	C1=0.
	C2=0.
	C3=0.
	D1=0.
	D2=0.
	D3=0.
      READ(1,*) (C1(I,I),C2(I,I),C3(I),D1(I,I),D2(I,I),D3(I),I=1,N)
C
C  Write the coefficients of the boundary conditions in the output file
C
      WRITE(2,500)
  500 FORMAT(//2X,'COEFFICIENTS OF THE BOUNDARY CONDITIONS'//2X,'NODE',
     # 15X,'c1',12X, 'c2',12X, 'c3',12X, 'd1',12X, 'd2',12X, 'd3')
      DO 30 I=1,N
   30 WRITE(2,600) I,C1(I,I),C2(I,I),C3(I),D1(I,I),D2(I,I),D3(I)
  600 FORMAT (2X,I3,5X,6E14.5)
C
C  Read the coordinates of the internal points
C
      READ(1,*) (XIN(I),YIN(I),I=1,IN)
C
C  Read the vectors FX,FY including the values of the body forces 
C     
      READ(1,*) (FX(I),FY(I),I=1,IN)
C      
C Write XIN, YIN and f in the output file      
C
      WRITE(2,700)
  700 FORMAT(//2X,'DOMAIN POINTS AND BODY FORCES'//
     1         9X,'XIN',8X,'YIN',12X,'Fx',12X,'Fy')
      DO 40 I=1,IN
   40 WRITE(2,800)XIN(I),YIN(I),FX(I),FY(I)
  800 FORMAT(2X,2(F10.2,2X),2E14.5)
C
C Read the coefficients of the constitutive matrix Eq. (8.151) and its 
C first derivatives at domain and internal nodal points and write them in the outputfile 
C
      DO 50 I=1,N
	READ (1,*) C11B(I),C12B(I),C13B(I),C22B(I),C23B(I),C33B(I)
   50 CONTINUE
      DO 60 I=1,IN
	READ(1,*) C11(I),C12(I),C13(I),C22(I),C23(I),C33(I)
   60 CONTINUE
      DO 70 I=1,IN
	READ(1,*) C11X(I),C12X(I),C13X(I),C22X(I),C23X(I),C33X(I)
   70 CONTINUE
      DO 80 I=1,IN
	READ(1,*) C11Y(I),C12Y(I),C13Y(I),C22Y(I),C23Y(I),C33Y(I)
   80 CONTINUE

  610 FORMAT(6(F16.8))
   	WRITE(2,900)
  900	FORMAT(//2X,'COEFFICIENTS OF THE ELASTIC CONSTITUTIVE MATRIX '/)
   	WRITE(2,1000)
 1000	FORMAT(/2X,'AT BOUDNARY POINTS '/8X,'C11',12X, 'C12',12X,'C13'
     # ,12X, 'C22',15X,'C23',12X, 'C33')

      DO 90 I=1,N
	WRITE (2,610) C11B(I),C12B(I),C13B(I),C22B(I),C23B(I),C33B(I)
   90 CONTINUE

   	WRITE(2,1100)
 1100	FORMAT(//2X,'AT DOMAIN NODAL POINTS'/8X,'C11',12X,'C12',12X,'C13'
     # ,12X, 'C22',15X,'C23',12X, 'C33')
      DO 110 I=1,IN
	WRITE (2,610) C11(I),C12(I),C13(I),C22(I),C23(I),C33(I)
  110 CONTINUE

     	WRITE(2,1150)
 1150	FORMAT(/2X,'FIRST DERIVATIVES OF THE COEFFICIENTS OF THE ELASTIC
     #CONSTITUTIVE MATRIX'/)

   	WRITE(2,1200)
 1200	FORMAT(/8X,'C11x',12X,'C12x',12X,'C13x',12X,'C22x',15X,'C23x',
     # 12X,'C33x')
      DO 120 I=1,IN
	WRITE (2,610) C11X(I),C12X(I),C13X(I),C22X(I),C23X(I),C33X(I)
  120 CONTINUE
   	WRITE(2,1300)
 1300	FORMAT(/8X,'C11y',12X,'C12y',12X,'C13y',12X,'C22y',15X,'C23y',
     # 12X,'C33y')
      DO 130 I=1,IN
	WRITE (2,610) C11Y(I),C12Y(I),C13Y(I),C22Y(I),C23Y(I),C33Y(I)
  130 CONTINUE
C
C Compute the finite difference matrix D of Eq. (8.83)
C
	D=0.
	K1=0
	DO K=1,NB

	IF (K.EQ.1) THEN
	N1=1
	N2=NL(1)
	ELSE
	N1=NL(K-1)+1
	N2=NL(K)
	ENDIF
	NN=N2-N1+1

	XL1=0.
	YL1=0.
	ICN1=0

	DO I=1,NN
	XL1(I)=XL(N1+I-1)
	YL1(I)=YL(N1+I-1)
	ICN1(I)=ICN(N1+I-1)
	ENDDO

	XL1(NN+1)=XL1(1)
	YL1(NN+1)=YL1(1)
	ICN1(NN+1)=ICN1(1)
     
      DT=0.
	CALL FINDIF(XL1,YL1,ICN1,NN,DT,N)

	DO I=1,NN
	DO J=1,NN
	D(I+K1,J+K1)=DT(I,J)
	ENDDO
	ENDDO
	K1=K1+NN
	ENDDO

      RETURN
      END   
C      
C==================================================================
C
      SUBROUTINE GMATR (XL,YL,XM,YM,G,N,NL,NB)
C
C  This subroutine computes the elements of the G matrix
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N),NL(NB)
      DIMENSION G(N,N)
C
C  Compute the boundary nodal coordinates and store them in  
C  the arrays XM and YM
C

      DO 10  I=1,N
      XM(I)=(XL(I)+XL(I+1))/2.
   10 YM(I)=(YL(I)+YL(I+1))/2.
	IF (NB.LE.1) GOTO 20
	XM(NL(1))=(XL(NL(1))+XL(1))/2.
	YM(NL(1))=(YL(NL(1))+YL(1))/2.
	DO 15 K=2,NB
	XM(NL(K))=(XL(NL(K))+XL(NL(K-1)+1))/2.
   15 YM(NL(K))=(YL(NL(K))+YL(NL(K-1)+1))/2.
C
C  Compute the elements of matrix G
C
   20 DO 30 I=1,N
      DO 30 J=1,N
      JP1=J+1
      IF(I.NE.J)THEN
      CALL RLINTC(XM(I),YM(I),XL(J),YL(J),XL(JP1),YL(JP1),RESULT)     
      G(I,J)=RESULT
      ELSEIF(I.EQ.J)THEN
      CALL SLINTC(XL(J),YL(J),XL(JP1),YL(JP1),RESULT)
      G(I,J)=RESULT
      ENDIF
   30 CONTINUE
      RETURN
      END
C
C
C================================================================
C
      SUBROUTINE RLINTC(X0,Y0,X1,Y1,X2,Y2,RESULT)
C
C
C  This subroutine computes the off-diagonal elelments of the 
C  matrix G 
C
C  RA= The distance of the point O from the Gauss integration point 
C      on the boundary element
C
C  WG= The weights of the Gauss integration
C
C  XI= The coordinates of the Gauss integration points in the 
C      interval [-1,1] 
C
C  XC,YC=The global coordinates of the Gauss integration points
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
      DIMENSION XC(4),YC(4),XI(4),WG(4)
      DATA XI/-0.86113631,-0.33998104,0.33998104,0.86113631/
      DATA WG/0.34785485,0.65214515,0.65214515,0.34785485/
      PI=ACOS(-1.)
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      BX=(X2+X1)/2.
      BY=(Y2+Y1)/2.
C
C  Compute the line integral 
C
      RESULT=0.
C                  
      DO 30  I=1,4
      XC(I)=AX*XI(I)+BX
      YC(I)=AY*XI(I)+BY
      RA=DSQRT((XC(I)-X0)**2+(YC(I)-Y0)**2)
   30 RESULT=RESULT+DLOG(RA)*WG(I)
      SL=2.*DSQRT(AX**2+AY**2)
      RESULT=RESULT*SL/(4.*PI)
      RETURN
      END
C
C
C=================================================================
C
      SUBROUTINE SLINTC(X1,Y1,X2,Y2,RESULT) 
C
C
C  This subroutines computes the diagonal elements of the matrix G
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
      PI=ACOS(-1.)
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      SL=DSQRT(AX**2+AY**2)
      RESULT=SL*(DLOG(SL)-1.)/PI
      RETURN
      END
C
C
C=================================================================
C												
      SUBROUTINE HMATR(XL,YL,XM,YM,H,N,NL,NB)
C
C
C  This subroutine computes the element of the H matrix
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)     
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N),NL(NB)
      DIMENSION H(N,N)
C
      PI=ACOS(-1.)
C
      DO 10  I=1,N
      XM(I)=(XL(I)+XL(I+1))/2.
   10 YM(I)=(YL(I)+YL(I+1))/2.
	IF (NB.LE.1) GOTO 20
	XM(NL(1))=(XL(NL(1))+XL(1))/2.
	YM(NL(1))=(YL(NL(1))+YL(1))/2.
	DO 15 K=2,NB
	XM(NL(K))=(XL(NL(K))+XL(NL(K-1)+1))/2.
   15 YM(NL(K))=(YL(NL(K))+YL(NL(K-1)+1))/2.

C
C  Compute the elements of H matrix
C
   20 DO 30  I=1,N
      DO 30  J=1,N
      IF (I.NE.J) THEN      
      CALL DALPHA (XM(I),YM(I),XL(J),YL(J),XL(J+1),YL(J+1),RESULT)
      H(I,J)=RESULT
      ELSEIF (I.EQ.J) THEN
      H(I,J)=-0.5
      ENDIF
   30 CONTINUE
      RETURN
      END
C
C
C=================================================================
C
      SUBROUTINE DALPHA(X0,Y0,X1,Y1,X2,Y2,RESULT)
C
C
C  This subroutine computes the off diagonal elements of the 
C  matrix H 
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
      PI=ACOS(-1.)
      DY1=Y1-Y0
      DX1=X1-X0
      DY2=Y2-Y0
      DX2=X2-X0
      DL1=DSQRT(DX1**2+DY1**2)
      COS1=DX1/DL1
      SIN1=DY1/DL1
      DX2R=DX2*COS1+DY2*SIN1
      DY2R=-DX2*SIN1+DY2*COS1
      DA=ATAN2(DY2R,DX2R)
      RESULT=DA/(2.*PI)
      RETURN
      END
C
C
C==================================================================
C
      SUBROUTINE UQTMATR (XL,YL,XM,YM,XIN,YIN,G,H,UHAT,QHAT,T,N,IN
	1   ,NL,NB,ENX,ENY)
C
C
C  This subroutine computes the matrices UHAT,QHAT and T
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
	COMMON C
      DIMENSION G(N,N),H(N,N),UHAT(N,IN),QHAT(N,IN),T(N,IN)
      DIMENSION XL(N+1),YL(N+1),XM(N),YM(N),XIN(IN),YIN(IN)
	DIMENSION NL(NB),ENX(N),ENY(N)
	
      DO 5  I=1,N
      XM(I)=(XL(I)+XL(I+1))/2.
    5 YM(I)=(YL(I)+YL(I+1))/2.

	IF (NB.LE.1) GOTO 10
	XM(NL(1))=(XL(NL(1))+XL(1))/2.
	YM(NL(1))=(YL(NL(1))+YL(1))/2.
	DO 15 K=2,NB
	XM(NL(K))=(XL(NL(K))+XL(NL(K-1)+1))/2.
   15 YM(NL(K))=(YL(NL(K))+YL(NL(K-1)+1))/2.
C
   10 WRITE(2,*)
	WRITE (*,*) 'MQs constant C :',C
	WRITE (2,*) 'MQs constant C :',C
C      
      DO 20 I=1,N
      DO 20 J=1,IN
      R=DSQRT((XM(I)-XIN(J))**2+(YM(I)-YIN(J))**2)
C
C  Computation of UHAT
C      
      UHAT(I,J)=-C**3/3.*DLOG(C*DSQRT(R**2+C**2)+C**2)+
	1               (R**2+4.*C**2)*DSQRT(R**2+C**2)/9.

C
C  Computation of QHAT
C
      QHAT(I,J)=1./(3.*DSQRT(R**2+C**2))*(R**2+2.*C**2-C**3/
	1  (DSQRT(R**2+C**2)+C))*((XM(I)-XIN(J))*ENX(I)
     1  +(YM(I)-YIN(J))*ENY(I))
   20 CONTINUE
C
C  Computation of T
C
	T=0.
	T=MATMUL(H,UHAT)-MATMUL(G,QHAT)

      RETURN
      END

C
C
C==================================================================
C
	SUBROUTINE BOUNDMATR(G,H,T,C1,C2,C3,D1,D2,D3,C11B,C12B,C13B
	1          ,C22B,C23B,C33B,ENX,ENY,D,N,IN,Z,E)
C
C  This subroutine computes the matrices Z,e
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)

      DIMENSION G(N,N),H(N,N),T(N,IN)
	DIMENSION ENX(N),ENY(N),D(N,N)
	DIMENSION C11B(N),C12B(N),C13B(N),C22B(N),C23B(N),C33B(N)
	DIMENSION C1(N,N),C2(N,N),C3(N),D1(N,N),D2(N,N),D3(N)


	DIMENSION Z(4*N,2*IN),E(4*N)
	DIMENSION A(4*N,4*N),AI(4*N,4*N)

	DIMENSION B11(N,N),B12(N,N),B13(N,N),B14(N,N)
	DIMENSION B21(N,N),B22(N,N),B23(N,N),B24(N,N)

	B11=0.
	B12=0.
	B13=0.
	B14=0.

	B21=0.
	B22=0.
	B23=0.
	B24=0.

	DO 10 I=1,N
	B11(I,I)=-(C11B(I)*ENX(I)+C13B(I)*ENY(I))*ENY(I)+(C13B(I)*ENX(I)
	1         +C33B(I)*ENY(I))*ENX(I)
	B12(I,I)=(C11B(I)*ENX(I)+C13B(I)*ENY(I))*ENX(I)+(C13B(I)*ENX(I)
	1         +C33B(I)*ENY(I))*ENY(I)
	B13(I,I)=-(C13B(I)*ENX(I)+C33B(I)*ENY(I))*ENY(I)+(C12B(I)*ENX(I)
	1         +C23B(I)*ENY(I))*ENX(I)
	B14(I,I)=(C13B(I)*ENX(I)+C33B(I)*ENY(I))*ENX(I)+(C12B(I)*ENX(I)
	1         +C23B(I)*ENY(I))*ENY(I)


	B21(I,I)=-(C13B(I)*ENX(I)+C12B(I)*ENY(I))*ENY(I)+(C33B(I)*ENX(I)
	1         +C23B(I)*ENY(I))*ENX(I)
	B22(I,I)=(C13B(I)*ENX(I)+C12B(I)*ENY(I))*ENX(I)+(C33B(I)*ENX(I)
	1         +C23B(I)*ENY(I))*ENY(I)
	B23(I,I)=-(C33B(I)*ENX(I)+C23B(I)*ENY(I))*ENY(I)+(C23B(I)*ENX(I)
	1         +C22B(I)*ENY(I))*ENX(I)
	B24(I,I)=(C33B(I)*ENX(I)+C23B(I)*ENY(I))*ENX(I)+(C23B(I)*ENX(I)
	1         +C22B(I)*ENY(I))*ENY(I)
   10	CONTINUE



	B11=MATMUL(B11,D)
	B11=MATMUL(C2,B11)
	B11=C1+B11

	B12=MATMUL(C2,B12)

	B13=MATMUL(B13,D)
	B13=MATMUL(C2,B13)

	B14=MATMUL(C2,B14)


	B21=MATMUL(B21,D)
	B21=MATMUL(D2,B21)

	B22=MATMUL(D2,B22)

	B23=MATMUL(D2,B23)
	B23=MATMUL(B23,D)
	B23=D1+B23

	B24=MATMUL(D2,B24)

  
	A=0.
	DO 20 I=1,N
	DO 20 J=1,N
	A(I,J)=H(I,J)
	A(I,J+N)=-G(I,J)

	A(I+N,J+2*N)=H(I,J)
	A(I+N,J+3*N)=-G(I,J)


	A(I+2*N,J)    =B11(I,J)
	A(I+2*N,J+N)  =B12(I,J)
	A(I+2*N,J+2*N)=B13(I,J)
	A(I+2*N,J+3*N)=B14(I,J)

	A(I+3*N,J)    =B21(I,J)
	A(I+3*N,J+N)  =B22(I,J)
	A(I+3*N,J+2*N)=B23(I,J)
	A(I+3*N,J+3*N)=B24(I,J)



   20 CONTINUE

	Z=0.
	E=0.
	DO 30 I=1,N
	E(I+2*N)=C3(I)
	E(I+3*N)=D3(I)

	DO 30 J=1,IN
	Z(I,J)=T(I,J)
	Z(I+N,J+IN)=T(I,J)
   30 CONTINUE

	AI=0.
	CALL DLINRG(4*N,A,4*N,AI,4*N)


	Z=MATMUL(AI,Z)
	E=MATMUL(AI,E)

	RETURN
	END
C
C===================================================================
C
      SUBROUTINE GHINMATR (XL,YL,XIN,YIN,GIN,GX,GY,GXX,GXY,GYY,
     1	                  HIN,HX,HY,HXX,HXY,HYY,ENX,ENY,N,IN,NL,NB)
C
C  This subroutine computes the INxN matrices Gin, Gx,Gy,...,Hin,..Hyy
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
 	COMMON C
      DIMENSION XL(N+1),YL(N+1),XIN(IN),YIN(IN),ENX(N),ENY(N),NL(NB)
      DIMENSION GIN(IN,N),GX(IN,N),GY(IN,N),GXX(IN,N),
     1          GXY(IN,N),GYY(IN,N)
      DIMENSION HIN(IN,N),HX(IN,N),HY(IN,N),HXX(IN,N),
     1          HXY(IN,N),HYY(IN,N)
      DIMENSION XC(4),YC(4),XI(4),WG(4)
C
C  WG       =Weights of the Gauss integration
C  XI       =Co-ordinates of of the Gauss integration points
C             in the interval [-1,1]
C  XC,YC    =Global co-ordinates of the gauss integration points
C
      DATA XI/-0.86113631,-0.33998104,0.33998104,0.86113631/
      DATA WG/0.34785485,0.65214515,0.65214515,0.34785485/

      PI=ACOS(-1.)
C

      DO 20 I=1,IN
      DO 20 J=1,N

      DO 10 IK=1,NB
	IF (J.EQ.NL(IK)) THEN
	 IF (IK.EQ.1) THEN
	   N2=1
	 ELSE
	   N2=NL(IK-1)+1
	 ENDIF
	ELSE
	  N2=J+1
   10	ENDIF
	
	N1=J
	AX=(XL(N2)-XL(N1))/2.
      AY=(YL(N2)-YL(N1))/2.
      BX=(XL(N2)+XL(N1))/2.
      BY=(YL(N2)+YL(N1))/2.
    
      SL=DSQRT(AX**2+AY**2)
C      
C Computation Gin, Gx,Gy,...,Hin,..Hyy  
C
         U=0.
         UX=0.
	   UY=0.
	   UXX=0.
	   UXY=0.
	   UYY=0.
         UN=0.
         UNX=0.
	   UNY=0.
	   UNXX=0.
	   UNXY=0.
	   UNYY=0.

      DO 40 IK=1,4
        XC(IK)=AX*XI(IK)+BX
        YC(IK)=AY*XI(IK)+BY
        DX=XC(IK)-XIN(I)
        DY=YC(IK)-YIN(I)
        R=DSQRT(DX**2+DY**2)
        RX=-DX/R
        RY=-DY/R
	  S=DLOG(R)/(2.*PI)	  
        SX=RX/(2.*PI*R)
        SY=RY/(2.*PI*R)
	  SXX=2.*PI*(SY**2-SX**2)
	  SYY=-SXX
	  SXY=-4.*PI*SX*SY

	  RN=-(RX*ENX(J)+RY*ENY(J))
        RT=-(-RX*ENY(J)+RY*ENX(J))

	  SN=RN/(2.*PI*R)
	  ST=RT/(2.*PI*R)
	  SNX=2.*PI*(SY*ST-SX*SN)
	  SNY=-2.*PI*(SX*ST+SY*SN)
	  SNXX=4.*PI*(SY*SNY-SX*SNX)
	  SNYY=-SNXX
	  SNXY=-4.*PI*(SY*SNX+SX*SNY)
C
        U=U+S*WG(IK)*SL
        UX=UX+SX*WG(IK)*SL
        UY=UY+SY*WG(IK)*SL
        UXX=UXX+SXX*WG(IK)*SL
        UXY=UXY+SXY*WG(IK)*SL
	  UYY=UYY+SYY*WG(IK)*SL
        UN=UN+SN*WG(IK)*SL
        UNX=UNX+SNX*WG(IK)*SL
	  UNY=UNY+SNY*WG(IK)*SL
	  UNXX=UNXX+SNXX*WG(IK)*SL
	  UNYY=UNYY+SNYY*WG(IK)*SL
	  UNXY=UNXY+SNXY*WG(IK)*SL
   40 CONTINUE
C
         GIN(I,J)=U
         GX(I,J)=UX
	   GY(I,J)=UY
	   GXX(I,J)=UXX
	   GXY(I,J)=UXY
	   GYY(I,J)=UYY
         HIN(I,J)=UN
         HX(I,J)=UNX
	   HY(I,J)=UNY
	   HXX(I,J)=UNXX
	   HXY(I,J)=UNXY
	   HYY(I,J)=UNYY
C
   20 CONTINUE
	RETURN
	END
C
C==================================================================
C
      SUBROUTINE UINHATMATR(XIN,YIN,UINHAT,UXHAT,UYHAT,
     1	                      UXXHAT,UXYHAT,UYYHAT,IN)
C
C  This subroutine computes the values of the matrices UHTIN,UHATX,
C  UHATY, UHATXX, UHATXY, UHATYY
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
	COMMON C
      DIMENSION XIN(IN),YIN(IN)
      DIMENSION UINHAT(IN,IN),UXHAT(IN,IN),UYHAT(IN,IN)
	DIMENSION UXXHAT(IN,IN),UXYHAT(IN,IN),UYYHAT(IN,IN)
C
      DO 10 I=1,IN
	DO 10 J=1,IN
C
C   Computation of UINHAT
C
      R=DSQRT((XIN(I)-XIN(J))**2+(YIN(I)-YIN(J))**2)
      UINHAT(I,J)=-C**3/3.*DLOG(C*DSQRT(R**2+C**2)+C**2)+
	1        (R**2+4.*C**2)*DSQRT(R**2+C**2)/9.
C   
C   Computation of UXHAT,UYHAT
C   
      UXHAT(I,J)=1./3./DSQRT(R**2+C**2)*(R**2+2.*C**2-C**3/
	1                          (DSQRT(R**2+C**2)+C))*(XIN(I)-XIN(J))
      UYHAT(I,J)=1./3./DSQRT(R**2+C**2)*(R**2+2.*C**2-C**3/
	1                          (DSQRT(R**2+C**2)+C))*(YIN(I)-YIN(J))
C
C   Computation of UXXHAT, UXYHAT, UYYHAT
C     
      IF(I.EQ.J) THEN
         UXXHAT(I,J)=C/2.
         UYYHAT(I,J)=C/2.
         UXYHAT(I,J)=0.
      ELSEIF(I.NE.J) THEN
	U1=1./3./(R**2+C**2)**(3./2.)*(R**2+C**3*(2.*DSQRT(R**2+C**2)+C)/
	1	  (DSQRT(R**2+C**2)+C)**2)
	U2=1./3./DSQRT(R**2+C**2)*(R**2+2.*C**2-C**3/
     1	  (DSQRT(R**2+C**2)+C))

         UXXHAT(I,J)=U1*(XIN(I)-XIN(J))**2+U2
         UYYHAT(I,J)=U1*(YIN(I)-YIN(J))**2+U2
         UXYHAT(I,J)=U1*(XIN(I)-XIN(J))*(YIN(I)-YIN(J))
      ENDIF
C
   10 CONTINUE   
      RETURN
      END
	

C
C=================================================================
C
	SUBROUTINE SZMATR(GIN,GX,GY,GXX,GXY,GYY,HIN,HX,HY,HXX,HXY,HYY
	1 ,UINHAT,UXHAT,UYHAT,UXXHAT,UXYHAT,UYYHAT,UHAT,QHAT,Z,E,N,IN
     1 ,SU1,SUX1,SUY1,SUXX1,SUXY1,SUYY1,SU2,SUX2,SUY2,SUXX2,SUXY2,SUYY2
     1 ,SV1,SVX1,SVY1,SVXX1,SVXY1,SVYY1,SV2,SVX2,SVY2,SVXX2,SVXY2,SVYY2
     1 ,ZU,ZUX,ZUY,ZUXX,ZUXY,ZUYY,ZV,ZVX,ZVY,ZVXX,ZVXY,ZVYY)
C
C  Compute the  matrices Su1,Su1x,Su1y,Su1xx,Su1xy,Su1yy
C                        Su2,Su2x,Su2y,Su2xx,Su2xy,Su2yy
C                        Sv1,Sv1x,Sv1y,Sv1xx,Sv1xy,Sv1yy
C                        Sv2,Sv2x,Sv2y,Sv2xx,Sv2xy,Sv2yy
C  and the vectors       zu,zux,zuy,zuxx,zuxy,zuyy
C                        zv,zvx,zvy,zvxx,zvxy,zvyy

      IMPLICIT DOUBLEPRECISION (A-H,O-Z)

      DIMENSION UHAT(N,IN),QHAT(N,IN)
      DIMENSION GIN(IN,N),GX(IN,N),GY(IN,N), 
     1          GXX(IN,N),GXY(IN,N),GYY(IN,N)
	DIMENSION HIN(IN,N),HX(IN,N),HY(IN,N),
     1          HXX(IN,N),HXY(IN,N),HYY(IN,N)
	DIMENSION UINHAT(IN,IN),UXHAT(IN,IN),UYHAT(IN,IN),
     1          UXXHAT(IN,IN),UXYHAT(IN,IN),UYYHAT(IN,IN)
	DIMENSION Z(4*N,2*IN),E(4*N),Z11(N,IN),Z12(N,IN),Z21(N,IN)
	1         ,Z22(N,IN),Z31(N,IN),Z32(N,IN),Z41(N,IN),Z42(N,IN)
	1	     ,E1(N),E2(N),E3(N),E4(N)
	DIMENSION S(IN,IN),SX(IN,IN),SY(IN,IN),SXX(IN,IN),SXY(IN,IN)
	1         ,SYY(IN,IN)

	DIMENSION SU1(IN,IN),SUX1(IN,IN),SUY1(IN,IN),SUXX1(IN,IN)
	1         ,SUXY1(IN,IN),SUYY1(IN,IN)
	DIMENSION SU2(IN,IN),SUX2(IN,IN),SUY2(IN,IN),SUXX2(IN,IN)
	1         ,SUXY2(IN,IN),SUYY2(IN,IN)
	DIMENSION ZU(IN),ZUX(IN),ZUY(IN),ZUXX(IN),ZUXY(IN),ZUYY(IN)

	DIMENSION SV1(IN,IN),SVX1(IN,IN),SVY1(IN,IN),SVXX1(IN,IN)
 	1         ,SVXY1(IN,IN),SVYY1(IN,IN)
	DIMENSION SV2(IN,IN),SVX2(IN,IN),SVY2(IN,IN),SVXX2(IN,IN)
	1         ,SVXY2(IN,IN),SVYY2(IN,IN)
	DIMENSION ZV(IN),ZVX(IN),ZVY(IN),ZVXX(IN),ZVXY(IN),ZVYY(IN)


	Z11=0.
	Z12=0.
	Z21=0.
	Z22=0.
	Z31=0.
	Z32=0.
	Z41=0.
	Z42=0.
	E1=0.
	E2=0.
	E3=0.
	E4=0.

	DO 10 I=1,N

	E1(I)=E(I)
	E2(I)=E(I+N)
	E3(I)=E(I+2*N)
	E4(I)=E(I+3*N)

	DO 10 J=1,IN
	Z11(I,J)=Z(I,J)
	Z12(I,J)=Z(I,J+IN)

	Z21(I,J)=Z(I+N,J)
	Z22(I,J)=Z(I+N,J+IN)

	Z31(I,J)=Z(I+2*N,J)
	Z32(I,J)=Z(I+2*N,J+IN)

	Z41(I,J)=Z(I+3*N,J)
	Z42(I,J)=Z(I+3*N,J+IN)

   10 CONTINUE

	S=0.
	S=UINHAT+MATMUL(GIN,QHAT)-MATMUL(HIN,UHAT)
	SX=0.
	SX=UXHAT+MATMUL(GX,QHAT)-MATMUL(HX,UHAT) 
	SY=0.
	SY=UYHAT+MATMUL(GY,QHAT)-MATMUL(HY,UHAT)
	SXX=0.
	SXX=UXXHAT+MATMUL(GXX,QHAT)-MATMUL(HXX,UHAT)
	SXY=0.
	SXY=UXYHAT+MATMUL(GXY,QHAT)-MATMUL(HXY,UHAT)
	SYY=0.
	SYY=UYYHAT+MATMUL(GYY,QHAT)-MATMUL(HYY,UHAT)


	SU1=0.
	SU1=S+MATMUL(HIN,Z11)-MATMUL(GIN,Z21)
	SU2=0.
	SU2=MATMUL(HIN,Z12)-MATMUL(GIN,Z22)
	ZU=0.
	ZU=MATMUL(HIN,E1)-MATMUL(GIN,E2)

	SUX1=0.
	SUX1=SX+MATMUL(HX,Z11)-MATMUL(GX,Z21)
	SUX2=0.
	SUX2=MATMUL(HX,Z12)-MATMUL(GX,Z22)
	ZUX=0.
	ZUX=MATMUL(HX,E1)-MATMUL(GX,E2)

	SUY1=0.
	SUY1=SY+MATMUL(HY,Z11)-MATMUL(GY,Z21)
	SUY2=0.
	SUY2=MATMUL(HY,Z12)-MATMUL(GY,Z22)
	ZUY=0.
	ZUY=MATMUL(HY,E1)-MATMUL(GY,E2)

	SUXX1=0.
 	SUXX1=SXX+MATMUL(HXX,Z11)-MATMUL(GXX,Z21)
	SUXX2=0.
	SUXX2=MATMUL(HXX,Z12)-MATMUL(GXX,Z22)
	ZUXX=0.
	ZUXX=MATMUL(HXX,E1)-MATMUL(GXX,E2)

	SUXY1=0.
 	SUXY1=SXY+MATMUL(HXY,Z11)-MATMUL(GXY,Z21)
	SUXY2=0.
	SUXY2=MATMUL(HXY,Z12)-MATMUL(GXY,Z22)
	ZUXY=0.
	ZUXY=MATMUL(HXY,E1)-MATMUL(GXY,E2)

	SUYY1=0.
 	SUYY1=SYY+MATMUL(HYY,Z11)-MATMUL(GYY,Z21)
	SUYY2=0.
	SUYY2=MATMUL(HYY,Z12)-MATMUL(GYY,Z22)
	ZUYY=0.
	ZUYY=MATMUL(HYY,E1)-MATMUL(GYY,E2)



	SV1=0.
	SV1=MATMUL(HIN,Z31)-MATMUL(GIN,Z41)
	SV2=0.
	SV2=S+MATMUL(HIN,Z32)-MATMUL(GIN,Z42)
	ZV=0.
	ZV=MATMUL(HIN,E3)-MATMUL(GIN,E4)

	SVX1=0.
	SVX1=MATMUL(HX,Z31)-MATMUL(GX,Z41)
	SVX2=0.
	SVX2=SX+MATMUL(HX,Z32)-MATMUL(GX,Z42)
	ZVX=0.
	ZVX=MATMUL(HX,E3)-MATMUL(GX,E4)

	SVY1=0.
	SVY1=MATMUL(HY,Z31)-MATMUL(GY,Z41)
	SVY2=0.
	SVY2=SY+MATMUL(HY,Z32)-MATMUL(GY,Z42)
	ZVY=0.
	ZVY=MATMUL(HY,E3)-MATMUL(GY,E4)

	SVXX1=0.
 	SVXX1=MATMUL(HXX,Z31)-MATMUL(GXX,Z41)
	SVXX2=0.
	SVXX2=SXX+MATMUL(HXX,Z32)-MATMUL(GXX,Z42)
	ZVXX=0.
	ZVXX=MATMUL(HXX,E3)-MATMUL(GXX,E4)

	SVXY1=0.
 	SVXY1=MATMUL(HXY,Z31)-MATMUL(GXY,Z41)
 	SVXY2=0.
	SVXY2=SXY+MATMUL(HXY,Z32)-MATMUL(GXY,Z42)
	ZVXY=0.
	ZVXY=MATMUL(HXY,E3)-MATMUL(GXY,E4)

	SVYY1=0.
 	SVYY1=MATMUL(HYY,Z31)-MATMUL(GYY,Z41)
	SVYY2=0.
	SVYY2=SYY+MATMUL(HYY,Z32)-MATMUL(GYY,Z42)
	ZVYY=0.
	ZVYY=MATMUL(HYY,E3)-MATMUL(GYY,E4)




	RETURN
	END
C
C==================================================================
C
	SUBROUTINE 	SOLVE(Z,E,SU1,SUX1,SUY1,SUXX1,SUXY1,SUYY1,SU2,SUX2
	1 ,SUY2,SUXX2,SUXY2,SUYY2,SV1,SVX1,SVY1,SVXX1,SVXY1,SVYY1,SV2
     1 ,SVX2,SVY2,SVXX2,SVXY2,SVYY2,ZU,ZUX,ZUY,ZUXX,ZUXY,ZUYY,ZV,ZVX
     1 ,ZVY,ZVXX,ZVXY,ZVYY,FX,FY,C11,C12,C13,C22,C23,C33,C11B,C12B
     1 ,C13B,C22B,C23B,C33B,C11X,C12X,C13X,C22X,C23X,C33X,C11Y,C12Y
     1 ,C13Y,C22Y,C23Y,C33Y,N,IN,ENX,ENY,D,U,UB,V,VB,SX,SY,SXY
     1 ,SXB,SYB,SXYB,STX,STY)
C
C  This subroutine solves the system of equations	K*A=P and computes the 
C  solution at internal and boundary points
C
      IMPLICIT DOUBLEPRECISION (A-H,O-Z)
	DIMENSION Z(4*N,2*IN),E(4*N),FX(IN),FY(IN),P(2*IN),A(2*IN)
	1         ,A1(IN),A2(IN),ENX(N),ENY(N),D(N,N)
	DIMENSION C11B(N),C12B(N),C13B(N),C22B(N),C23B(N),C33B(N)
 	DIMENSION C11(IN),C12(IN),C13(IN),C22(IN),C23(IN),C33(IN)
	DIMENSION C11X(IN),C12X(IN),C13X(IN),C22X(IN),C23X(IN),C33X(IN)
	DIMENSION C11Y(IN),C12Y(IN),C13Y(IN),C22Y(IN),C23Y(IN),C33Y(IN)

	DIMENSION SU1(IN,IN),SUX1(IN,IN),SUY1(IN,IN),SUXX1(IN,IN)
 	1         ,SUXY1(IN,IN),SUYY1(IN,IN)
	DIMENSION SU2(IN,IN),SUX2(IN,IN),SUY2(IN,IN),SUXX2(IN,IN)
	1         ,SUXY2(IN,IN),SUYY2(IN,IN)
	DIMENSION ZU(IN),ZUX(IN),ZUY(IN),ZUXX(IN),ZUXY(IN),ZUYY(IN)

	DIMENSION SV1(IN,IN),SVX1(IN,IN),SVY1(IN,IN),SVXX1(IN,IN)
 	1         ,SVXY1(IN,IN),SVYY1(IN,IN)
	DIMENSION SV2(IN,IN),SVX2(IN,IN),SVY2(IN,IN),SVXX2(IN,IN)
	1         ,SVXY2(IN,IN),SVYY2(IN,IN)
	DIMENSION ZV(IN),ZVX(IN),ZVY(IN),ZVXX(IN),ZVXY(IN),ZVYY(IN)

	DIMENSION UVB(4*N)
      DIMENSION U(IN),UX(IN),UY(IN),UXX(IN),UXY(IN),UYY(IN)
	DIMENSION UB(N),UNB(N),UTB(N),UXB(N),UYB(N)
      DIMENSION V(IN),VX(IN),VY(IN),VXX(IN),VXY(IN),VYY(IN)
 	DIMENSION VB(N),VNB(N),VTB(N),VXB(N),VYB(N)
	DIMENSION SX(IN),SY(IN),SXY(IN),SXB(N),SYB(N),SXYB(N)
	1         ,STX(N),STY(N)

	DOUBLEPRECISION K(2*IN,2*IN)

C
C  Formulate system of equations
C
	K=0.
	P=0.

	DO 10 I=1,IN

	P(I)=FX(I)-(C11(I)*ZUXX(I)+2.*C13(I)*ZUXY(I)+C33(I)*ZUYY(I)
 	1 +(C11X(I)+C13Y(I))*ZUX(I)+(C13X(I)+C33Y(I))*ZUY(I)
     1 +C13(I)*ZVXX(I)+(C12(I)+C33(I))*ZVXY(I)+C23(I)*ZVYY(I)
	1 +(C13X(I)+C33Y(I))*ZVX(I)+(C12X(I)+C23Y(I))*ZVY(I))

	P(I+IN)=FY(I)-(C13(I)*ZUXX(I)+(C12(I)+C33(I))*ZUXY(I)
	1 +C23(I)*ZUYY(I)
 	1 +(C13X(I)+C12Y(I))*ZUX(I)+(C33X(I)+C23Y(I))*ZUY(I)
     1 +C33(I)*ZVXX(I)+2.*C23(I)*ZVXY(I)+C22(I)*ZVYY(I)
	1 +(C33X(I)+C23Y(I))*ZVX(I)+(C23X(I)+C22Y(I))*ZVY(I))

	DO 10 J=1,IN

	K(I,J)=C11(I)*SUXX1(I,J)+2.*C13(I)*SUXY1(I,J)+C33(I)*SUYY1(I,J)
	1 +(C11X(I)+C13Y(I))*SUX1(I,J)+(C13X(I)+C33Y(I))*SUY1(I,J)
     1 +C13(I)*SVXX1(I,J)+(C12(I)+C33(I))*SVXY1(I,J)+C23(I)*SVYY1(I,J)
	1 +(C13X(I)+C33Y(I))*SVX1(I,J)+(C12X(I)+C23Y(I))*SVY1(I,J)

	K(I,J+IN)=C11(I)*SUXX2(I,J)+2.*C13(I)*SUXY2(I,J)+C33(I)*SUYY2(I,J)
	1 +(C11X(I)+C13Y(I))*SUX2(I,J)+(C13X(I)+C33Y(I))*SUY2(I,J)
     1 +C13(I)*SVXX2(I,J)+(C12(I)+C33(I))*SVXY2(I,J)+C23(I)*SVYY2(I,J)
	1 +(C13X(I)+C33Y(I))*SVX2(I,J)+(C12X(I)+C23Y(I))*SVY2(I,J)
	K(I+IN,J)=C13(I)*SUXX1(I,J)+(C12(I)+C33(I))*SUXY1(I,J)
	1 +C23(I)*SUYY1(I,J)
 	1 +(C13X(I)+C12Y(I))*SUX1(I,J)+(C33X(I)+C23Y(I))*SUY1(I,J)
     1 +C33(I)*SVXX1(I,J)+2.*C23(I)*SVXY1(I,J)+C22(I)*SVYY1(I,J)
	1 +(C33X(I)+C23Y(I))*SVX1(I,J)+(C23X(I)+C22Y(I))*SVY1(I,J)
	K(I+IN,J+IN)=C13(I)*SUXX2(I,J)+(C12(I)+C33(I))*SUXY2(I,J)
	1 +C23(I)*SUYY2(I,J)
 	1 +(C13X(I)+C12Y(I))*SUX2(I,J)+(C33X(I)+C23Y(I))*SUY2(I,J)
     1 +C33(I)*SVXX2(I,J)+2.*C23(I)*SVXY2(I,J)+C22(I)*SVYY2(I,J)
	1 +(C33X(I)+C23Y(I))*SVX2(I,J)+(C23X(I)+C22Y(I))*SVY2(I,J)


   10 CONTINUE
C
C  Solve linear system of equations K*X=P
C
      CALL LEQS(K,P,2*IN,LSING)
      IF(LSING.EQ.0)THEN
      WRITE(2,150)
  150 FORMAT(/,'',69('*')//2X'The system has been solved regularly'/)
      ELSEIF(LSING.EQ.1)THEN
      WRITE(2,200)
  200 FORMAT(/,'',69('*')//2X'The system is singular'/)
      ENDIF
	A=P
	A1=0.
	A2=0.
	DO 20 I=1,IN
	A1(I)=A(I)
	A2(I)=A(I+IN)
   20 CONTINUE
C
C  Compute u,ux,uy,uxx,uyy and uxy at the internal points
C

	U=0.
	U=MATMUL(SU1,A1)+MATMUL(SU2,A2)+ZU
	UX=0.
	UX=MATMUL(SUX1,A1)+MATMUL(SUX2,A2)+ZUX
	UY=0.
	UY=MATMUL(SUY1,A1)+MATMUL(SUY2,A2)+ZUY
	UXX=0.
	UXX=MATMUL(SUXX1,A1)+MATMUL(SUXX2,A2)+ZUXX
	UXY=0.
	UXY=MATMUL(SUXY1,A1)+MATMUL(SUXY2,A2)+ZUXY
	UYY=0.
	UYY=MATMUL(SUYY1,A1)+MATMUL(SUYY2,A2)+ZUYY

C
C  Compute v,vx,vy,vxx,vyy and vxy at the internal points
C
	V=0.
      V=MATMUL(SV1,A1)+MATMUL(SV2,A2)+ZV
	VX=0.
	VX=MATMUL(SVX1,A1)+MATMUL(SVX2,A2)+ZVX
	VY=0.
	VY=MATMUL(SVY1,A1)+MATMUL(SVY2,A2)+ZVY
	VXX=0.
	VXX=MATMUL(SVXX1,A1)+MATMUL(SVXX2,A2)+ZVXX
	VXY=0.
	VXY=MATMUL(SVXY1,A1)+MATMUL(SVXY2,A2)+ZVXY
	VYY=0.
	VYY=MATMUL(SVYY1,A1)+MATMUL(SVYY2,A2)+ZVYY
C
C  Compute stresses sx.sy.sxy at internal points
C
	DO 30 I=1,IN
	SX(I)=C11(I)*UX(I)+C12(I)*VY(I)+C13(I)*(UY(I)+VX(I))
	SY(I)=C12(I)*UX(I)+C22(I)*VY(I)+C23(I)*(UY(I)+VX(I))
	SXY(I)=C13(I)*UX(I)+C23(I)*VY(I)+C33(I)*(UY(I)+VX(I))
   30 CONTINUE

C
C  Compute u,un,v,vn at boundary points
C   

	UVB=0.
	UVB=MATMUL(Z,P)+E

	DO 40 I=1,N
	UB(I)=UVB(I)
	UNB(I)=UVB(I+N)
	VB(I)=UVB(I+2*N)
	VNB(I)=UVB(I+3*N)
   40 CONTINUE
C
C  Compute stresses sxb,syb,sxyb and tractions stx,sty at boundary points
C

	UTB=0.
	UTB=MATMUL(D,UB)
	VTB=0.
	VTB=MATMUL(D,VB)
	STX=0.
	STY=0.
	SXB=0.
	SYB=0.
	SXYB=0.

	DO 50 I=1,N
	UXB(I)=ENX(I)*UNB(I)-ENY(I)*UTB(I)
	UYB(I)=ENY(I)*UNB(I)+ENX(I)*UTB(I)
	VXB(I)=ENX(I)*VNB(I)-ENY(I)*VTB(I)
	VYB(I)=ENY(I)*VNB(I)+ENX(I)*VTB(I)


	SXB(I)=C11B(I)*UXB(I)+C12B(I)*VYB(I)+C13B(I)*(UYB(I)+VXB(I))
	SYB(I)=C12B(I)*UXB(I)+C22B(I)*VYB(I)+C23B(I)*(UYB(I)+VXB(I))
	SXYB(I)=C13B(I)*UXB(I)+C23B(I)*VYB(I)+C33B(I)*(UYB(I)+VXB(I))

	STX(I)=SXB(I)*ENX(I)+SXYB(I)*ENY(I)
	STY(I)=SXYB(I)*ENX(I)+SYB(I)*ENY(I)
   50 CONTINUE
	RETURN
	END
C
C==================================================================
C
      SUBROUTINE LEQS(A,B,N,LSING)
C
C   This subroutine uses Gauss elimination to solve
C   a system of linear equations, [A]{X}={B}, where
C   A : One-dimensional array which contains the occasional row of
C       the two-dimensional array of the coefficients of the unknowns
C   B : One-dimensional array which contains the known coefficients
C   N : Integer denoting the number of the unknowns
C   LSING: Integer taking the values:
C         LSING = 0, if the system has been solved regularly
C         LSING = 1, if the system is singular
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(1)
C
      LSING=0
      JJ=-N
      DO 10 J=1,N
        JY=J+1
        JJ=JJ+N+1
        AMAX=0.0
        IHELP=JJ-J
      DO 20 I=J,N
        IJ=IHELP+I
        IF(ABS(AMAX)-ABS(A(IJ))) 30,20,20
   30      AMAX=A(IJ)
           IMAX=I
   20 CONTINUE
         IF(ABS(AMAX).EQ.0.) THEN
            LSING=1
            RETURN
         END IF
      I1=J+N*(J-2)
      IHELP=IMAX-J
      DO 40 K=J,N
        I1=I1+N
        I2=I1+IHELP
        ATEMP=A(I1)
        A(I1)=A(I2)
        A(I2)=ATEMP
   40 A(I1)=A(I1)/AMAX
      ATEMP=B(IMAX)
      B(IMAX)=B(J)
      B(J)=ATEMP/AMAX
         IF(J-N) 50,70,50
   50       IQS=N*(J-1)
            DO 10 IX=JY,N
              IXJ=IQS+IX
              IHELP=J-IX
              DO 60 JX=JY,N
                IJREF=N*(JX-1)+IX
                JJX=IJREF+IHELP
   60           A(IJREF)=A(IJREF)-(A(IXJ)*A(JJX))
   10 B(IX)=B(IX)-B(J)*A(IXJ)
   70    NY=N-1
      NN=N*N
      DO 80 J=1,NY
        I1=NN-J
        I2=N-J
        I3=N
      DO 80 K=1,J
        B(I2)=B(I2)-A(I1)*B(I3)
        I1=I1-N
   80   I3=I3-1
      RETURN
      END
C
C==========================================================
C
	SUBROUTINE FINDIF(XL,YL,ICN,N,D,NT) 
C
C
C  This subroutine computes the finite difference matrix of Eq.(8.83)
C
	IMPLICIT DOUBLEPRECISION (A-H,O-Z)                     
      DIMENSION XL(N+1),YL(N+1),D(NT,NT),ICN(N+1)
	N1=1
	N2=N
	NM=N
	D=0.

	DO 100 I=1,NM
	X2=XL(N1+I-1)
	Y2=YL(N1+I-1)
      X3=XL(N1+I)
	Y3=YL(N1+I)
	sl2=0.
	SL2=DSQRT((X3-X2)**2+(Y3-Y2)**2)
	IF (I.EQ.1.OR.ICN(N1+I-1).EQ.1) THEN
C	Forward finite difference
	X4=XL(N1+I+1)
 	Y4=YL(N1+I+1)
	X5=XL(N1+I+2)
	Y5=YL(N1+I+2)
	sl3=0.
	sl4=0.
	SL3=DSQRT((X3-X4)**2+(Y3-Y4)**2)
	SL4=DSQRT((X4-X5)**2+(Y4-Y5)**2)
	S1=(sL2+SL3)/2.
	S2=(sL3+SL4)/2.
	A1=-(2*S1+S2)/S1/(S1+S2)
	A2=(S1+S2)/S1/S2
	A3=-S1/S2/(S1+S2)
	D(I,I)=A1
	D(I,I+1)=A2
	D(I,I+2)=A3
	ELSEIF (I.EQ.Nm.OR.ICN(N1+I).EQ.1) THEN
C	Backward finite difference
	X0=XL(N1+I-3)
	Y0=YL(N1+I-3)
	X1=XL(N1+I-2)
	Y1=YL(N1+I-2)
	sl0=0.
	sl1=0.
	SL0=DSQRT((X0-X1)**2+(Y0-Y1)**2)
	SL1=DSQRT((X1-X2)**2+(Y1-Y2)**2)
	S1=(sL2+SL1)/2.
	S2=(sL1+SL0)/2.
	A1=(2*S1+S2)/S1/(S1+S2)
	A2=-(S1+S2)/S1/S2
	A3=S1/S2/(S1+S2)
	D(I,I-2)=A3
	D(I,I-1)=A2
	D(I,I)=A1
	ELSE
C	Central finite difference
	X1=XL(N1+I-2)
	Y1=YL(N1+I-2)
	X4=XL(N1+I+1)
	Y4=YL(N1+I+1)
	sl1=0.
	sl3=0.
	SL1=DSQRT((X1-X2)**2+(Y1-Y2)**2)
	SL3=DSQRT((X3-X4)**2+(Y3-Y4)**2)
	S1=(sL1+SL2)/2.
	S2=(sL3+SL2)/2.
	A1=-S2/S1/(S1+S2)
	A2=-(S1-S2)/S1/S2
	A3=S1/S2/(S1+S2)
	D(I,I-1)=A1
	D(I,I)=A2
	D(I,I+1)=A3
	ENDIF

  100 CONTINUE

	RETURN
	END
C	              
C==================================================================
C
      SUBROUTINE OUTPUT(XM,YM,XIN,YIN,U,UB,V,VB,SX,SY
	1 ,SXY,SXB,SYB,SXYB,STX,STY,N,IN)
C
C  This subroutine prints the results in the outputfile.
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XM(N),YM(N),XIN(IN),YIN(IN)
      DIMENSION U(IN),UB(N),V(IN),VB(N)
	DIMENSION SX(IN),SY(IN),SXY(IN),SXB(N),SYB(N),SXYB(N)
	1         ,STX(N),STY(N)
C
      WRITE(2,100)
  100 FORMAT('',69('*')//1X,'RESULTS'//1X,'DISPLACEMENTS AT BOUNDARY NOD
     1ES'//1X,'NODE',11X,'X',15X,'Y',12X,'U',15X,'V'/)

      DO 10 I=1,N
	X=XM(I)
	Y=YM(I)        
      WRITE(2,200) I,XM(I),YM(I),UB(I),VB(I)
   10 CONTINUE 
  200 FORMAT(I4,5(2X,E14.8))
C
      WRITE (2,300)
  300 FORMAT(//,1X,'DISPLCAMENTS AT INTERNAL POINTS'//1X,'NODE',10X,'X'
     1,15X,'Y',15X,'U',14X,'V',/)
      
      DO 20 I=1,IN
      WRITE(2,400)  I,XIN(I),YIN(I),U(I),V(I)
   20	CONTINUE 
  400 FORMAT(I4,4(2X,E14.8))

	WRITE(2,500)
  500 FORMAT(//,1X,'STRESSES AT BOUNDARY NODAL POINTS'/)
      WRITE(2,600)
  600 FORMAT(1X,'NODE',9X,'X',15X,'Y',13X,'SX',15X,'SY',13X,'SXY'/)
      DO 30 I=1,N
   30 WRITE(2,700)I,XM(I),YM(I),SXB(I),SYB(I),SXYB(I)
  700 FORMAT(I4,5(2X,E14.8))

      WRITE(2,800)
  800 FORMAT(//,1X,'TRACTIONS AT BOUNDARY NODAL POINTS'/)

       WRITE(2,900)
  900 FORMAT(1X,'NODE',7X,'X',14X,'Y',17X,'TX',15X,'TY'/)
 	DO 40 I=1,N
 	WRITE (2,400) I,XM(I),YM(I),STX(I),STY(I)
   40 CONTINUE 
 
      WRITE(2,1000)
 1000 FORMAT(//,1X,'STRESSES AT INTERNAL POINTS'/)

      WRITE(2,600)
      DO 50 I=1,IN
 	X=XIN(I)
 	Y=YIN(I)
   50 WRITE(2,700)  I,XIN(I),YIN(I),SX(I),SY(I),SXY(I) 
 
      WRITE(2,1100)
 1100 FORMAT('',84('*'))


      RETURN
      END          
