% Script file: aemelbedata.m
%
% Purpose:
%    This M-file prapares the data for the program AEMELBE   

% Example 8.6

clear all;
fprintf('********* AEMELBE DATA ********** \n');
inputfile1=input('Enter file name :','s');
fid = fopen (inputfile1,'w');
user=input('User name:','s');
title=input('Title:','s');
%==========================================================================
% BOUNDARY DATA
%==========================================================================
nb=1;                   % number of boundaries
                             %
ns=4;                   % number of sides.
                            % A side is defined as a part of the boundary  
                            % between the corner points. Points where
                            % the boundary conditions change or points 
                            % of point supports treated as corner points.
                            % Parts of the boundary with different geometry
                            % are different sides.

% Give the side properties
%ATENSION: External boundaries are run  anticlockwise,
%          internal boundaries clockwise. 

%---------------------------------ms-----------------------------------------------          
%                         1            2    3        4       5         6             7       8        
%------------------------------------------------------------------------------------                       
%line      : 1 | No of BE | x1 | y1  |   x2   |  y2   | unused   |  unused |    
%            
%circle   : 2 | No of BE | x0 | y0  | phi1 | phi2 | radious   | unused   | 
%          
%ellipse:  3 | No of BE | x0 | y0  | phi1 | ph2  | radious1 | radious2|            
%---------------------------------------------------------                                                              
% Give: 1 for straight line; 2 for circular arc
% x1               : x coordinate  of the starting point
% y1               : y coordinate  of the starting point
% x2               : x coordinate  of the end point
% y2               : y coordinate  of the end point

% x0               : x coordinate  of the center 
% y0               : y coordinate  of the center
% phi1, phi2       : limit angles of the circular arc
% radious1 : lenght of semiaxis 1
% radious2 : lenght of semiaxis 2
% 
% ms:= array including the side properteis
ms=zeros(ns,7);
ms=[ 1  ,99, -1.0 ,-0.5 , 1.0 ,-0.5 ,0,0;
         1  , 50,  1.0, -0.5 , 1.0 ,0.5 ,0,0;
         1  , 99,  1.0, 0.5 ,-1.0 ,0.5,0 ,0;
         1  , 50,  -1.0 ,0.5,-1.0, -0.5 ,0 ,0
         ];
%       
%  CALL function: boundarypoints
%                For the meanning of the output parameters see function
%                boundarypoints.m
nl=zeros(nb,1);
nl=[298];               % serial number of the last element of each boundary

[nbe,xb,yb,xl,yl,jn,kn,angle,cn,sn,length,cur]=boundarypoints(ms,ns,nl);

 % Boundary conditions for each side:
 % **********************************************************************
 % c1, c2, c3, d1, d2, d3 vectors with the coefficients of the boundary conditions
 % **********************************************************************           
  %           code     value 1   value 2
 %  bc =[    1     ,      u      ,      v   ;
 %              2     ,      tx     ,      ty  ;
 %              3     ,      u      ,      ty  ;
 %              4     ,      tx     ,      v   ]
 
 
 bc=zeros(ns,3);
  bc= [4,0,0;
          2,100,0;
          4,0,0;
          2,-100,0];
      
c1=zeros(nb,1);
c2=zeros(nb,1);
c3=zeros(nb,1);

ik=0;
for i=1:ns
    for j=1:ms(i,2)
        ik=ik+1;
        x=xb(ik);
        y=yb(ik);
        if bc(i,1) == 1
            
            c1(ik)=1;
            c2(ik)=0;
            c3(ik)=bc(i,2);
            d1(ik)=1;
            d2(ik)=0;
            d3(ik)=bc(i,3);
            
        elseif bc(i,1) == 2
            c1(ik)=0;
            c2(ik)=1;
            c3(ik)=bc(i,2);
            d1(ik)=0;
            d2(ik)=1;
            d3(ik)=bc(i,3);
        elseif bc(i,1) == 3
            c1(ik)=1;
            c2(ik)=0;
            c3(ik)=bc(i,2);
            d1(ik)=0;
            d2(ik)=1;
            d3(ik)=bc(i,3);
        elseif bc(i,1) == 4
            c1(ik)=0;
            c2(ik)=1;
            c3(ik)=bc(i,2);
            d1(ik)=1;
            d2(ik)=0;
            d3(ik)=bc(i,3);
        end
        
        if abs(xb(ik)) < 0.0001
            c1(ik)=1;
            c2(ik)=0;
            c3(ik)=0;
        end
    end
       
    
end


%==========================================================================  
% GENERATION of the coorinates of the domain mesh points xin,yin where the
% solution and its derivatives  will be evaluated.
% A rectangular mesh with nrow X ncol nodal points is applied on the domain
% Then the function checkin.m detects the nodal points
% inside the domain. The function checkdist detects interior nodal points 
% at distance from the boundary smaller than a minimum prescribed distance 
% and excludes them.
%==========================================================================
nrow=10;
ncol=20;
inpoint=nrow*ncol;
ax=max(xb)-min(xb);
by=max(yb)-min(yb);
dxin=ax/(ncol+1);
dyin=by/(nrow+1);
nrow1=nrow;
ncol1=ncol;
mt=nrow1*ncol1;       
xint=zeros(1,mt);
yint=zeros(1,mt);

for i=1:nrow1
    for j=1:ncol1
        k=(i-1)*ncol1+j;
       xint(k)=min(xb)+j*dxin;
       yint(k)=min(yb)+i*dyin;
    end
end
%==========================================================================
% CALL function checkin.m to detect points inside the domain of the plate
%==========================================================================
dist=0.01;  %the minimum distance of the internal points from the boundary
[m,xin,yin]=checkin(nbe,mt,xb,yb,xint,yint,cn,sn,length,dist);
          % m  = final total number of mesh point points
%==========================================================================
% COEFFICIENTS of the Equation and RIGHT HAND SIDE of the Equation 
%==========================================================================
for i=1:m
    fx(i)=0;
    fy(i)=0;
end
for i=1:m
    x=xin(i);
    y=yin(i);
   
    c11(i)=800000*(10-9*x^2)/(8*(10-9*x^2)*(1+9*x^2)-5);
    c12(i)=200000.00*(10-9*x^2)/(8*(10-9*x^2)*(1+9*x^2)-5);
    c13(i)=0;
    c22(i)=80000.0*(10-9*x^2)^2*(1+9*x^2)/(8*(10-9*x^2)*(1+9*x^2)-5);
    c23(i)=0;
    c33(i)=40000*1/(10+9*x^2);
    
    c11x(i)=-14400000*x/(8*(10-9*x^2)*(1+9*x^2)-5) ...
        -800000*(10-9*x^2)*(-144*x*(1+9*x^2) ...
        +144*(10-9*x^2)*x)/((8*(10-9*x^2)*(1+9*x^2)-5)^2);
    c12x(i)=-3600000.00*x/(8*(10-9*x^2)*(1+9*x^2)-5) ...
        -200000.00*(10-9*x^2)*(-144*x*(1+9*x^2) ...
        +144*(10-9*x^2)*x)/((8*(10-9*x^2)*(1+9*x^2)-5)^2);
    c13x(i)=0;
    c22x(i)=-2880000.0*(10-9*x^2)*(1+9*x^2)*x/(8*(10-9*x^2)*(1+9*x^2)-5) ...
        +1440000.0*(10-9*x^2)^2*x/(8*(10-9*x^2)*(1+9*x^2)-5) ...
        -80000.0*(10-9*x^2)^2*(1+9*x^2)*(-144*x*(1+9*x^2) ...
        +144*(10-9*x^2)*x)/((8*(10-9*x^2)*(1+9*x^2)-5)^2);
    c23x(i)=0;
    c33x(i)=-720000*x/((10+9*x^2)^2);
    
    c11y(i)=0;
    c12y(i)=0;
    c13y(i)=0;
    c22y(i)=0;
    c23y(i)=0;
    c33y(i)=0;
end
for i=1:nbe
    x=xb(i);
    y=yb(i);
    c11b(i)=800000*(10-9*x^2)/(8*(10-9*x^2)*(1+9*x^2)-5);
    c12b(i)=200000.00*(10-9*x^2)/(8*(10-9*x^2)*(1+9*x^2)-5);
    c13b(i)=0;
    c22b(i)=80000.0*(10-9*x^2)^2*(1+9*x^2)/(8*(10-9*x^2)*(1+9*x^2)-5);
    c23b(i)=0;
    c33b(i)=40000*1/(10+9*x^2);
end

%==========================================================================
% PLOT DATA
%==========================================================================
fprintf(' No of boudary elements \n');
nbe
fprintf(' No of internal  points \n');
m

plot(xl,yl,'m+',xb,yb,'b.',xin,yin,'ro');  
%==========================================================================
% WRITE data in the inputfile
%==========================================================================
fprintf(fid,user );
fprintf(fid,'\n');
fprintf(fid,title );
fprintf(fid,'\n');
for ii=1:nbe
    fprintf(fid,'%6.8f %6.8f %0.0f\n',xl(ii),yl(ii),angle(ii));
end
for ii=1:nbe
    fprintf(fid,'%5.5f %5.5f %5.5f %5.5f %5.5f %5.5f\n',c1(ii),c2(ii),c3(ii),d1(ii),d2(ii),d3(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f %5.5f \n',xin(ii),yin(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f  %5.5f \n',fx(ii),fy(ii));
end
for ii=1:nbe
    fprintf(fid,'%5.5f %5.5f %5.5f %5.5f %5.5f %5.5f \n', ... 
        c11b(ii),c12b(ii),c13b(ii),c22b(ii),c23b(ii),c33b(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f %5.5f %5.5f %5.5f %5.5f %5.5f \n', ... 
        c11(ii),c12(ii),c13(ii),c22(ii),c23(ii),c33(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f %5.5f %5.5f %5.5f %5.5f %5.5f \n', ... 
        c11x(ii),c12x(ii),c13x(ii),c22x(ii),c23x(ii),c33x(ii));
end
for ii=1:m
    fprintf(fid,'%5.5f %5.5f %5.5f %5.5f %5.5f %5.5f \n', ... 
        c11y(ii),c12y(ii),c13y(ii),c22y(ii),c23y(ii),c33y(ii));
end




fclose(fid);

